#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# pinger4.py
#
# Dieses Script prüft den Online-Status der unter HOSTS_TO_PING
# angegebenen Hosts. Statt ping wird das hier besser geeignete fping
# verwendet.
# Parallel werden auch die IP-Adressen aus VLAN_PING über Bande (bzw.
# ein System, das Teilnehmer in VLAN1 und VLAN10 ist) geprüft. Hier
# werden die IP-Adressen zusätzlich durch die im Dictionary definierten
# Hostnames substituiert.
# Änderungen am Online-Status werden zusammen mit dem jew.
# Timestamp in die Datei gemäß LOGFILE geschrieben.
#
# D.Ahlgrimm    21.05.2022
#
# D.A.  28.05.2022      v4      Umbau auf arp-Nutzung

import subprocess
import time
import os
import sys

LOGFILE="/home/dede/log_pinger.txt"

HOSTS_TO_PING=[ "fb", "switch", "8.8.8.8", "gw2", "nffw", "dlna", "ss",
                "btv2", "c12", "c2q", "i5",
                "rs",  "rp1", "rp2", "rp3", "t22", "bb",
                "ct1", "lrp2", "bp1", "bp2",
              ]

ARP_INTERFACE="eth0.10"
VLAN_NET="192.168.86.0/24"
VLAN_PING={ "B0:6A:41:DC:6F:C7" : "GWifi-Büro",
            "B0:6A:41:DC:6E:51" : "GWifi-Wohn",
            "B0:6A:41:DC:67:19" : "GWifi-Schlaf",
            "08:B4:B1:79:76:2B" : "GWifi-Dach",
            "64:66:B3:FA:BA:C8" : "nffw1",
            "84:16:F9:9B:AC:E4" : "nffw4",
            "0C:CB:85:5D:69:B2" : "MotoG6+",
            "A8:13:74:88:7A:1D" : "GlotzeSZ",
            "A8:13:74:4B:D3:B0" : "GlotzeWZ",

          }
VLAN_IP_TO_MAC=dict()   # Format: { "IP" : "MAC" }

TIME_FORMAT="%Y%m%d-%H%M%S"


# ----------------------------------------------------------------------
# Pingt alle Hosts aus HOSTS_TO_PING und liefert die
# Hostnamen als Liste zurück, die online sind.
#
# Exit status of fping is 
# 0 if all the hosts are reachable, 
# 1 if some hosts were unreachable, 
# 2 if any IP addresses were not found, 
# 3 for invalid command line arguments, and 
# 4 for a system call failure.
def ping_normal():
    cmd_l=["fping", "-a"] + HOSTS_TO_PING
    process=subprocess.Popen(cmd_l, shell=False, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
    rc=process.communicate()[0]

    ret_lst=set()
    if process.returncode<3:
        for ln in rc.decode("utf8").split("\n"):
            if ln.strip()!="":
                ret_lst.add(ln.strip())
    else:
        print("rc=", process.returncode)
    return(ret_lst)


# ----------------------------------------------------------------------
# Wie ping_normal() - nur über den Umweg via DLNA-VM bzw. VLAN10.
def ping_vlan10():
    cmd="fping -a " + " ".join(VLAN_IP_TO_MAC.keys())
    cmd_l=["ssh", "dlna", cmd]
    process=subprocess.Popen(cmd_l, shell=False, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
    rc=process.communicate()[0]

    ret_lst=set()
    if process.returncode<3:
        for ln in rc.decode("utf8").split("\n"):
            ip=ln.strip()
            if ip!="" and ip in VLAN_IP_TO_MAC:
                mac=VLAN_IP_TO_MAC[ip]
                if mac in VLAN_PING:
                    ret_lst.add(VLAN_PING[mac])
    else:
        print("rc=", process.returncode)
    return(ret_lst)

    
# ----------------------------------------------------------------------
# Pingt jede IP-Adresse in VLAN_NET einmal an, um den arp-Cache zu
# füllen/aktualisieren.
def refresh_arp():
    cmd="fping -q -c 1 -g "+VLAN_NET
    cmd_l=["ssh", "dlna", cmd]
    process=subprocess.Popen(cmd_l, shell=False, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
    rc=process.communicate()[0]


# ----------------------------------------------------------------------
# Aktualisiert das Dictionary VLAN_IP_TO_MAC mit der jüngsten Version
# von der Beziehung "IP-Adresse zu MAC-Adresse".
def get_arp():
    refresh_arp()
    cmd="sudo arp -i "+ARP_INTERFACE
    cmd_l=["ssh", "dlna", cmd]
    process=subprocess.Popen(cmd_l, shell=False, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
    rc=process.communicate()[0]

    if process.returncode<3:
        for ln in rc.decode("utf8").split("\n"):
            lnl=ln.split()
            # Liefert etwa sowas hier:
            #    ['Address', 'HWtype', 'HWaddress', 'Flags', 'Mask', 'Iface']
            #    ['192.168.86.62', '(unvollständig)', 'eth0.10']
            #    ['192.168.86.41', 'ether', '3e:98:e5:3a:19:d6', 'C', 'eth0.10']
            #    ['192.168.86.22', 'ether', 'b0:6a:41:dc:67:19', 'C', 'eth0.10']
            #    ....
            #    []
            if len(lnl)>=3:
                ip=lnl[0]
                mac=lnl[2].upper()
                if mac in VLAN_PING:
                    del_ips=[]
                    for i, m in VLAN_IP_TO_MAC.items():
                        if m==mac and i!=ip:                # wenn die MAC schon drin ist, aber eine andere IP zugeordnet hat
                            del_ips.append(i)               # zum Löschen vormerken
                    for del_ip in del_ips:                  # über alle Vormerkungen
                        del VLAN_IP_TO_MAC[del_ip]          # weg damit
                    VLAN_IP_TO_MAC.update({ip:mac})


# ----------------------------------------------------------------------
# Schreibt "txt" in die Datei gemäß LOGFILE.
def logStatus(txt):
    #print(txt)
    #return
    fobj=open(LOGFILE, "a")
    fobj.write(txt+"\n")
    fobj.close()


# ----------------------------------------------------------------------
#
if __name__=='__main__':
    get_arp()
    onlineHosts=ping_normal() | ping_vlan10()
    olh=", ".join(sorted(onlineHosts))
    tm=time.strftime(TIME_FORMAT)

    logStatus("%s restart "%(tm,) + "-"*50)
    logStatus("%s online  %s"%(tm, olh))

    while True:
        time.sleep(60)
        tm=time.strftime(TIME_FORMAT)
        get_arp()
        newOnlineHosts=ping_normal() | ping_vlan10()

        online=", ".join(sorted(newOnlineHosts-onlineHosts))
        offline=", ".join(sorted(onlineHosts-newOnlineHosts))
        if len(online)>0:
            logStatus("%s online  %s"%(tm, online))
        if len(offline)>0:
            logStatus("%s offline %s"%(tm, offline))
        onlineHosts=newOnlineHosts
